﻿using System;
using System.Linq;
using Composite.Data;
using Composite.Data.Types;

namespace CustomMediaFileProvider
{
    public class MediaFileFolder: IMediaFileFolder
    {
        private readonly DataSourceId _dataSourceId;
        private readonly string _keyPath;
        private readonly string _compositePath;

        public MediaFileFolder(string path, string storeId, Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaDataId dataId, string providerName)
        {
            string[] pathParts = path.Split('/');

            _dataSourceId = new DataSourceId(dataId, providerName, typeof(IMediaFileFolder));

            Id = dataId.Id;
            Description = "";
            Title = pathParts.Last();
            StoreId = storeId;
            Path = path;

            _keyPath = storeId + ":" + Id;
            _compositePath = storeId + ":" + path;
        }

        public Guid Id
        {
            get;
            private set;
        }

        public string KeyPath
        {
            get { return _keyPath; }
        }

        public string CompositePath
        {
            get { return _compositePath; }
            set { throw new NotImplementedException(); }
        }

        public string StoreId
        {
            get;
            set;
        }

        public string Path
        {
            get;
            set;
        }

        public string Title
        {
            get;
            set;
        }

        public string Description
        {
            get;
            set;
        }

        public bool IsReadOnly
        {
            get;
            set;
        }

        public DataSourceId DataSourceId
        {
            get { return _dataSourceId; }
        }
    }
}
