﻿using System;
using System.Linq;
using Composite.Core.Routing;
using Composite.Core.WebClient.Media;
using Composite.Data.Types;

namespace CustomMediaFileProvider
{
    public class MediaUrlProvider : IMediaUrlProvider
    {
        private const int DEFAULT_HEIGHT = 300;
        private const int DEFAULT_WIDTH = 300;

        private readonly MediaFileProvider _mediaFileProvider;
        private readonly ResizingOptions _defaultOptions;

        public MediaUrlProvider(MediaFileProvider mediaFileProvider,ResizingOptions defaultOptions = null)
        {
            _mediaFileProvider = mediaFileProvider;
            _defaultOptions = defaultOptions;
        }

        public bool IsSupportedStoreId(string storeId)
        {
            return _mediaFileProvider.StoreId == storeId;
        }

        public string GetPublicMediaUrl(string storeId, Guid mediaId)
        {
            return GetResizedImageUrl(storeId, mediaId, null);
        }


        public string GetResizedImageUrl(string storeId, Guid mediaId, ResizingOptions resizingOptions)
        {
            var file = _mediaFileProvider.GetData<IMediaFile>().FirstOrDefault(f => f.Id == mediaId) as UrlBasedMediaFile;

            if (file == null)
            {
                return null;
            }

            string downloadUrl = file.DownloadUrl;

            int height = DEFAULT_HEIGHT;
            int width = DEFAULT_WIDTH;


            if (_defaultOptions != null && !_defaultOptions.IsEmpty)
            {
                height = _defaultOptions.MaxHeight??DEFAULT_HEIGHT;
                width = _defaultOptions.MaxWidth??DEFAULT_WIDTH;
            }
            
            if (resizingOptions != null && !resizingOptions.IsEmpty)
            {
                if (resizingOptions.MaxWidth != null)
                {
                    width = resizingOptions.MaxWidth.Value;
                }

                if (resizingOptions.MaxHeight != null)
                {
                    height = resizingOptions.MaxHeight.Value;
                }
            }

            return downloadUrl.Replace("{width}", width.ToString()).Replace("{height}", height.ToString());
        }
    }

    public class ResizableMediaUrlProvider : MediaUrlProvider,IResizableImageUrlProvider
    {
        public ResizableMediaUrlProvider(MediaFileProvider mediaFileProvider,ResizingOptions defaultOptions = null) : base(mediaFileProvider,defaultOptions)
        {
        }
    }
}
