﻿using System;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using Composite.Core.Configuration;
using Composite.Core.IO;
using Composite.Core.WebClient;
using Composite.Data.Streams;
using Composite.Data.Types;

namespace CustomMediaFileProvider
{
    class UrlBasedFileStreamManager: IFileStreamManager
    {
        private static readonly MD5 HashingAlgorithm = MD5.Create();

        private static readonly object _downloadLock = new object();

        public Stream GetReadStream(IFile file)
        {
            var urlBasedMedia = (UrlBasedMediaFile) file;
            string tempFileName = GetTempFileName(urlBasedMedia);

            string downloadUrl = urlBasedMedia.DownloadUrl.Replace("{width}", "500").Replace("{height}", "500");

            if (!File.Exists(tempFileName))
            {
                lock (_downloadLock)
                {
                    if (!File.Exists(tempFileName))
                    {
                        using (var client = new WebClient())
                        {
                            client.DownloadFile(downloadUrl, tempFileName);
                        }
                    }
                }
            }

            return File.OpenRead(tempFileName);
        }

        private static string GetTempFileName(UrlBasedMediaFile file)
        {
            var tempDir = PathUtil.Resolve(GlobalSettingsFacade.TempDirectory);
            string hash = UrlUtils.CompressGuid(GetHashValue(file.DownloadUrl));
            string extension = Path.GetFileName(file.FileName);

            return Path.Combine(tempDir, "urlBasedMedia" + hash + "." + extension);
        }

        public Stream GetNewWriteStream(IFile file)
        {
            throw new NotSupportedException();
        }

        public void SubscribeOnFileChanged(IFile file, OnFileChangedDelegate handler)
        {
        }

        private static Guid GetHashValue(string value)
        {
            var bytes = HashingAlgorithm.ComputeHash(Encoding.UTF8.GetBytes(value));
            return new Guid(bytes);
        }
    }
}
