﻿using System;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using Composite.Core.IO;
using Composite.Data;
using Composite.Data.Streams;
using Composite.Data.Types;

namespace CustomMediaFileProvider
{
    [FileStreamManager(typeof(UrlBasedFileStreamManager))]
    public sealed class UrlBasedMediaFile: IMediaFile
    {
        private static readonly MD5 HashingAlgorithm = MD5.Create();

        public UrlBasedMediaFile(
            string downloadUrl, 
            string fileName, 
            string folderName, 
            string storeId, 
            int length,
            DateTime creationTime,
            DateTime lastWriteTime,
            DataSourceId dataSourceId)
        {
            Id = CalculateId(folderName, fileName);
            DownloadUrl = downloadUrl;
            FileName = fileName;
            FolderPath = folderName;
            StoreId = storeId;
            DataSourceId = dataSourceId;
            CreationTime = creationTime;
            LastWriteTime = lastWriteTime;
            Length = length;
        }

        internal static Guid CalculateId(string folderName, string fileName)
        {
            return GetHashValue(folderName + "/" + fileName);
        }

        private static Guid GetHashValue(string value)
        {
            var bytes = HashingAlgorithm.ComputeHash(Encoding.ASCII.GetBytes(value));
            return new Guid(bytes);
        }


        public Guid Id { get; internal set; }

        public string DownloadUrl { get; internal set; }

        public string KeyPath
        {
            get { return this.GetKeyPath();  }
        }


        public string CompositePath
        {
            get { return this.GetCompositePath(); }
            set { throw new NotImplementedException(); }
        }


        public string StoreId
        {
            get;
            set;
        }



        public string Title
        {
            get
            {
                return this.FileName;
            }
            set
            {
                this.FileName = value;
            }
        }



        public string Description
        {
            get
            {
                return "";
            }
            set
            {
            }
        }



        public string Culture
        {
            get
            {
                return CultureInfo.InvariantCulture.Name;
            }
            set
            {
            }
        }



        public string MimeType
        {
            get { return MimeTypeInfo.GetCanonicalFromExtension(Path.GetExtension(this.FileName)); }
        }




        public int? Length { get; set; }
        

        public DateTime? CreationTime { get; set; }
        

        public DateTime? LastWriteTime { get; set; }


        public bool IsReadOnly
        {
            get
            {
                return true;
            }
            set
            {
            }
        }



        public string FolderPath
        {
            get; 
            set;
        }



        public string FileName
        {
            get;
            set;
        }



        public DataSourceId DataSourceId
        {
            get;
            private set;
        }
    }
}
